{***************************************************************************}
{                              UNIDAD DE LISTA

                                 Copyright (c) 1998 Marco Antonio Alvarado
}
{***************************************************************************}

UNIT ULista;

INTERFACE

  TYPE
    PPiezaLista = ^OPiezaLista;

    OPiezaLista = OBJECT
      Siguiente : PPiezaLista;
      CONSTRUCTOR Inicia;
      DESTRUCTOR Termina; VIRTUAL;
    END;

    OLista = OBJECT
      Primera,
      Indice : PPiezaLista;
      CONSTRUCTOR Inicia;
      DESTRUCTOR Termina; VIRTUAL;
      FUNCTION EstaVacia : Boolean; VIRTUAL;
      FUNCTION TomaUltima : PPiezaLista; VIRTUAL;
      PROCEDURE Agrega (Nueva : PPiezaLista); VIRTUAL;
      PROCEDURE SueltaUltima; VIRTUAL;
      PROCEDURE EliminaUltima; VIRTUAL;
      FUNCTION CuentaLugar (Hasta : PPiezaLista) : Word; VIRTUAL;
      FUNCTION TomaLugar (Lugar : Word) : PPiezaLista; VIRTUAL;
      PROCEDURE IniciaIndice; VIRTUAL;
      PROCEDURE AvanzaIndice; VIRTUAL;
    END;

IMPLEMENTATION

  CONSTRUCTOR OPiezaLista. Inicia;
  BEGIN
    Siguiente := NIL;
  END;

  DESTRUCTOR OPiezaLista. Termina;
  BEGIN
  END;

{***************************************************************************}

  CONSTRUCTOR OLista. Inicia;
  BEGIN
    Primera := NIL;
    Indice := NIL;
  END;

  DESTRUCTOR OLista. Termina;
  BEGIN
    WHILE EstaVacia = False DO EliminaUltima;
  END;

  FUNCTION OLista. EstaVacia : Boolean;
  BEGIN
    EstaVacia := Primera = NIL;
  END;

  FUNCTION OLista. TomaUltima : PPiezaLista;

    VAR
      Rastro : PPiezaLista;

  BEGIN

    IF EstaVacia THEN
    BEGIN
      TomaUltima := NIL;
    END ELSE
    BEGIN
      Rastro := Primera;
      WHILE Rastro^. Siguiente <> NIL DO Rastro := Rastro^. Siguiente;
      TomaUltima := Rastro;
    END;

  END;

  PROCEDURE OLista. Agrega (Nueva : PPiezaLista);
  BEGIN

    IF EstaVacia THEN
    BEGIN
      Primera := Nueva;
    END ELSE
    BEGIN
      TomaUltima^. Siguiente := Nueva;
    END;

  END;

  PROCEDURE OLista. SueltaUltima;

    VAR
      Vieja,
      Rastro : PPiezaLista;

  BEGIN

    IF EstaVacia = False THEN
    BEGIN
      Vieja := TomaUltima;
      Rastro := Primera;

      IF Rastro = Vieja THEN
      BEGIN
        Primera := NIL;
      END ELSE
      BEGIN
        WHILE Rastro^. Siguiente <> Vieja DO Rastro := Rastro^. Siguiente;
        Rastro^. Siguiente := NIL;
      END;

    END;

  END;

  PROCEDURE OLista. EliminaUltima;

    VAR
      Vieja : PPiezaLista;

  BEGIN
    Vieja := TomaUltima;
    SueltaUltima;
    Dispose (Vieja, Termina);
  END;

  FUNCTION OLista. CuentaLugar (Hasta : PPiezaLista) : Word;

    VAR
      Rastro : PPiezaLista;
      Lugar : Word;

  BEGIN

    IF EstaVacia THEN
    BEGIN
      CuentaLugar := 0;
    END ELSE
    BEGIN
      Rastro := Primera;
      Lugar := 1;

      WHILE Rastro <> Hasta DO
      BEGIN
        Rastro := Rastro^. Siguiente;
        Inc (Lugar);
      END;

      CuentaLugar := Lugar;
    END;

  END;

  FUNCTION OLista. TomaLugar (Lugar : Word) : PPiezaLista;

    VAR
      Rastro : PPiezaLista;

  BEGIN

    IF EstaVacia THEN
    BEGIN
      TomaLugar := NIL;
    END ELSE
    BEGIN

      IF Lugar = 1 THEN
      BEGIN
        TomaLugar := Primera;
      END ELSE
      BEGIN
        Rastro := Primera^. Siguiente;

        FOR Lugar := Lugar DOWNTO 2 DO
        BEGIN
          IF Rastro = NIL THEN Break;
          Rastro := Rastro^. Siguiente;
        END;

        TomaLugar := Rastro;
      END;

    END;

  END;

  PROCEDURE OLista. IniciaIndice;
  BEGIN
    Indice := Primera;
  END;

  PROCEDURE OLista. AvanzaIndice;
  BEGIN
    IF Indice <> NIL THEN Indice := Indice^. Siguiente;
  END;

END.